# Generated by Django 4.2.7 on 2023-11-28 14:49

from django.db import migrations, models
import django.db.models.deletion
import picklefield.fields


class Migration(migrations.Migration):
    dependencies = [
        ("djwebdapp", "0016_alter_transaction_has_code"),
    ]

    operations = [
        migrations.CreateModel(
            name="Event",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=500)),
                (
                    "args",
                    picklefield.fields.PickledObjectField(
                        blank=True, default=dict, editable=False
                    ),
                ),
                ("event_index", models.PositiveBigIntegerField()),
                ("normalized", models.BooleanField(default=False)),
                (
                    "transaction",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="transactionevent_set",
                        to="djwebdapp.transaction",
                    ),
                ),
            ],
        ),
    ]
