# Generated by Django 4.1.1 on 2023-02-09 14:23

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("djwebdapp", "0013_transaction_normalized_dependency"),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name="account",
            unique_together={("blockchain", "address")},
        ),
    ]
