# Generated by Django 4.1.1 on 2023-02-03 14:36

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("djwebdapp_tezos", "0003_tezoscall_tezoscontract"),
    ]

    operations = [
        migrations.DeleteModel(
            name="TezosCall",
        ),
        migrations.DeleteModel(
            name="TezosContract",
        ),
    ]
