# Generated by Django 4.0.5 on 2022-06-27 13:20

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djwebdapp', '0006_blockchain_min_confirmations_alter_transaction_state'),
    ]

    operations = [
        migrations.AlterField(
            model_name='transaction',
            name='amount',
            field=models.DecimalField(blank=True, db_index=True, decimal_places=18, default=0, max_digits=80),
        ),
    ]
