# Generated by Django 4.1a1 on 2022-09-08 17:50

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("djwebdapp_tezos", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="tezostransaction",
            name="caller",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="_internal_calls",
                to="djwebdapp_tezos.tezostransaction",
            ),
        ),
    ]
