# Generated by Django 4.0.5 on 2022-06-17 13:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djwebdapp', '0005_alter_transaction_unique_together_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='blockchain',
            name='min_confirmations',
            field=models.PositiveIntegerField(default=2, help_text='Number of confirmation blocks from confirm to done states'),
        ),
        migrations.AlterField(
            model_name='transaction',
            name='state',
            field=models.CharField(choices=[('held', 'Held'), ('deleted', 'Deleted during reorg'), ('aborted', 'Aborted'), ('deploy', 'To deploy'), ('deploying', 'Deploying'), ('retry', 'To retry'), ('retrying', 'Retrying'), ('confirm', 'Deployed to confirm'), ('done', 'Confirmed finished')], db_index=True, default='held', max_length=200),
        ),
    ]
