# Generated by Django 4.2.1 on 2023-05-11 16:23

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("djwebdapp", "0015_alter_transaction_state"),
    ]

    operations = [
        migrations.AlterField(
            model_name="transaction",
            name="has_code",
            field=models.BooleanField(
                default=False,
                help_text="Checked if this transaction has smart contract code.",
            ),
        ),
    ]
