# Generated by Django 5.2 on 2025-04-15 17:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djwebdapp', '0017_event'),
    ]

    operations = [
        migrations.AddField(
            model_name='blockchain',
            name='chain_id',
            field=models.IntegerField(blank=True, null=True),
        ),
    ]
