# Generated by Django 4.1.1 on 2022-09-11 17:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("djwebdapp", "0009_account_index"),
    ]

    operations = [
        migrations.AlterField(
            model_name="transaction",
            name="nonce",
            field=models.IntegerField(default=-1),
        ),
    ]
