# Generated by Django 5.2 on 2025-11-20 11:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("djwebdapp", "0019_alter_event_event_index_alter_event_normalized_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="account",
            name="address",
            field=models.CharField(
                blank=True, db_index=True, max_length=255, null=True
            ),
        ),
    ]
