# Generated by Django 4.1.1 on 2022-09-11 16:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("djwebdapp", "0007_alter_transaction_amount"),
    ]

    operations = [
        migrations.AddField(
            model_name="transaction",
            name="index",
            field=models.BooleanField(
                default=True,
                help_text="Wether the indexer should index all transactions or not",
            ),
        ),
    ]
