# Generated by Django 3.2.18 on 2023-04-08 13:33

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djwebdapp_ethereum', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='EthereumCall',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djwebdapp_ethereum.ethereumtransaction',),
        ),
        migrations.CreateModel(
            name='EthereumContract',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djwebdapp_ethereum.ethereumtransaction',),
        ),
    ]
