# Generated by Django 3.2.18 on 2023-04-08 19:20

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djwebdapp_ethereum', '0002_ethereumcall_ethereumcontract'),
    ]

    operations = [
        migrations.AddField(
            model_name='ethereumtransaction',
            name='caller',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='_internal_calls', to='djwebdapp_ethereum.ethereumtransaction'),
        ),
    ]
