# Generated by Django 3.2.18 on 2023-04-11 14:08

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('djwebdapp_tezos', '0005_tezoscall_tezoscontract'),
        ('djwebdapp', '0015_alter_transaction_state'),
    ]

    operations = [
        migrations.CreateModel(
            name='FA12Tezos',
            fields=[
                ('tezostransaction_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='djwebdapp_tezos.tezostransaction')),
                ('token_name', models.CharField(max_length=200)),
                ('token_symbol', models.CharField(max_length=10)),
            ],
            bases=('djwebdapp_tezos.tezoscontract',),
        ),
        migrations.CreateModel(
            name='FA12TezosMint',
            fields=[
                ('tezostransaction_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='djwebdapp_tezos.tezostransaction')),
                ('mint_amount', models.PositiveIntegerField()),
                ('mint_account', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='djwebdapp.account')),
                ('target_contract', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='djwebdapp_example_tezos.fa12tezos')),
            ],
            bases=('djwebdapp_tezos.tezoscall',),
        ),
        migrations.CreateModel(
            name='FA12TezosBalance',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('balance', models.PositiveIntegerField(default=0)),
                ('account', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='djwebdapp.account')),
                ('fa12', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='djwebdapp_example_tezos.fa12tezos')),
            ],
            options={
                'unique_together': {('fa12', 'account')},
            },
        ),
    ]
