# Generated by Django 4.1.1 on 2022-09-16 14:34

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("djwebdapp", "0010_alter_transaction_nonce"),
    ]

    operations = [
        migrations.RenameField(
            model_name="blockchain",
            old_name="max_level",
            new_name="index_level",
        ),
    ]
