# Generated by Django 4.2.7 on 2023-11-28 14:49

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("djwebdapp", "0017_event"),
        ("djwebdapp_ethereum", "0003_ethereumtransaction_caller"),
    ]

    operations = [
        migrations.CreateModel(
            name="EthereumEvent",
            fields=[
                (
                    "event_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="djwebdapp.event",
                    ),
                ),
                (
                    "contract",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="contractevent_set",
                        to="djwebdapp_ethereum.ethereumcontract",
                    ),
                ),
            ],
            bases=("djwebdapp.event",),
        ),
    ]
