# Generated by Django 5.2 on 2025-11-13 17:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("djwebdapp", "0018_blockchain_chain_id"),
    ]

    operations = [
        migrations.AlterField(
            model_name="event",
            name="event_index",
            field=models.PositiveBigIntegerField(db_index=True),
        ),
        migrations.AlterField(
            model_name="event",
            name="normalized",
            field=models.BooleanField(db_index=True, default=False),
        ),
        migrations.AlterField(
            model_name="transaction",
            name="created_at",
            field=models.DateTimeField(auto_now_add=True, db_index=True, null=True),
        ),
        migrations.AlterField(
            model_name="transaction",
            name="hash",
            field=models.CharField(
                blank=True, db_index=True, max_length=255, null=True
            ),
        ),
        migrations.AlterField(
            model_name="transaction",
            name="nonce",
            field=models.IntegerField(db_index=True, default=-1),
        ),
        migrations.AlterField(
            model_name="transaction",
            name="normalized",
            field=models.BooleanField(
                db_index=True,
                default=False,
                help_text="Enabled when transaction is normalized",
            ),
        ),
    ]
